use 5.006;
use strict;
use warnings;
use Module::Build;
Module::Build->VERSION('0.4004');

my $builder = Module::Build->new(
    module_name         => 'SMS::API::VoIP::MS',
    license             => 'artistic_2',
    dist_author         => q{Justin Wheeler <cpan@datademons.com>},
    dist_version_from   => 'lib/SMS/API/VoIP/MS.pm',
    release_status      => 'stable',

    configure_requires => {
        'Module::Build' => '0.4004',
    },

    test_requires => {
        'Test::More' => '0',
    },

    requires => {
        'POSIX'              => 0,
        'Date::Parse'        => 0,
        'MIME::Base64'       => 0,
        'IO::Scalar'         => 0,
        'Image::Magick'      => 0,
        Mo                   => 0,
        URI                  => 0,
        'LWP::UserAgent'     => 0,
        'Cpanel::JSON::XS'   => 0,
        'Unicode::LineBreak' => 0,
        'Scalar::Util'       => 0,
    },

    add_to_cleanup => [ 'SMS-API-VoIP-MS-*' ],
);

$builder->create_build_script();
